@@ -36,7 +36,22 @@
 #define FLEXDEF_H 1
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
+#endif
+
+/* AIX requires this to be the first thing in the file.  */
+#ifndef __GNUC__
+# if HAVE_ALLOCA_H
+#  include <alloca.h>
+# else
+#  ifdef _AIX
+ #pragma alloca
+#  else
+#   ifndef alloca /* predefined by HP cc +Olibcalls */
+char *alloca ();
+#   endif
+#  endif
+# endif
 #endif
 
 #ifdef STDC_HEADERS
@@ -48,6 +63,12 @@
 #include <string.h>
 #include <math.h>
 #endif
+#ifdef HAVE_ASSERT_H
+#include <assert.h>
+#else
+#define assert(Pred)
+#endif
+
 #ifdef HAVE_LIMITS_H
 #include <limits.h>
 #endif
@@ -70,7 +91,9 @@
 #define true 1
 #define false 0
 #endif
+#ifdef HAVE_REGEX_H
 #include <regex.h>
+#endif
 #include "flexint.h"
 
 /* We use gettext. So, when we write strings which should be translated, we mark them with _() */
@@ -94,7 +117,7 @@
 #endif
 
 #ifndef PROTO
-#if __STDC__
+#if defined(__STDC__)
 #define PROTO(proto) proto
 #else
 #define PROTO(proto) ()
@@ -202,7 +225,7 @@
 #define JAM -1			/* to mark a missing DFA transition */
 #define NO_TRANSITION NIL
 #define UNIQUE -1		/* marks a symbol as an e.c. representative */
-#define INFINITY -1		/* for x{5,} constructions */
+#define INFINITE_REPEAT -1		/* for x{5,} constructions */
 
 #define INITIAL_MAX_CCLS 100	/* max number of unique character classes */
 #define MAX_CCLS_INCREMENT 100
@@ -257,7 +280,7 @@
 
 /* The percentage the number of homogeneous out-transitions of a state
  * must be of the number of total out-transitions of the state in order
- * that the state's transition table is first compared with a potential 
+ * that the state's transition table is first compared with a potential
  * template of the most common out-transition instead of with the first
  * proto in the proto queue.
  */
@@ -335,7 +358,6 @@
  * nowarn - if true (-w), do not generate warnings
  * spprdflt - if true (-s), suppress the default rule
  * interactive - if true (-I), generate an interactive scanner
- * caseins - if true (-i), generate a case-insensitive scanner
  * lex_compat - if true (-l), maximize compatibility with AT&T lex
  * posix_compat - if true (-X), maximize compatibility with POSIX lex
  * do_yylineno - if true, generate code to maintain yylineno
@@ -377,7 +399,7 @@
 
 extern int printstats, syntaxerror, eofseen, ddebug, trace, nowarn,
 	spprdflt;
-extern int interactive, caseins, lex_compat, posix_compat, do_yylineno;
+extern int interactive, lex_compat, posix_compat, do_yylineno;
 extern int useecs, fulltbl, usemecs, fullspd;
 extern int gen_line_dirs, performance_report, backing_up_report;
 extern int reentrant, bison_bridge_lval, bison_bridge_lloc;
@@ -394,7 +416,6 @@ extern int yymore_really_used, reject_really_used;
  * dataline - number of contiguous lines of data in current data
  * 	statement.  Used to generate readable -f output
  * linenum - current input line number
- * out_linenum - current output line number
  * skelfile - the skeleton file
  * skel - compiled-in skeleton array
  * skel_ind - index into "skel" array, if skelfile is nil
@@ -410,7 +431,7 @@ extern int yymore_really_used, reject_really_used;
  * use_stdout - the -t flag
  * input_files - array holding names of input files
  * num_input_files - size of input_files array
- * program_name - name with which program was invoked 
+ * program_name - name with which program was invoked
  *
  * action_array - array to hold the rule actions
  * action_size - size of action_array
@@ -422,13 +443,13 @@ extern int yymore_really_used, reject_really_used;
  * 	to "action_array"
  */
 
-extern int datapos, dataline, linenum, out_linenum;
+extern int datapos, dataline, linenum;
 extern FILE *skelfile, *yyin, *backing_up_file;
 extern const char *skel[];
 extern int skel_ind;
 extern char *infilename, *outfilename, *headerfilename;
 extern int did_outfilename;
-extern char *prefix, *yyclass;
+extern char *prefix, *yyclass, *extra_type;
 extern int do_stdinit, use_stdout;
 extern char **input_files;
 extern int num_input_files;
@@ -719,6 +740,8 @@ extern int yylval;
 extern void ccladd PROTO ((int, int));	/* add a single character to a ccl */
 extern int cclinit PROTO ((void));	/* make an empty ccl */
 extern void cclnegate PROTO ((int));	/* negate a ccl */
+extern int ccl_set_diff (int a, int b); /* set difference of two ccls. */
+extern int ccl_set_union (int a, int b); /* set union of two ccls. */
 
 /* List the members of a set of characters in CCL form. */
 extern void list_character_set PROTO ((FILE *, int[]));
@@ -817,7 +840,7 @@ extern void usage PROTO ((void));
 extern void action_define PROTO ((const char *defname, int value));
 
 /* Add the given text to the stored actions. */
-extern void add_action PROTO ((char *new_text));
+extern void add_action PROTO ((const char *new_text));
 
 /* True if a string is all lower case. */
 extern int all_lower PROTO ((register char *));
@@ -825,8 +848,8 @@ extern int all_lower PROTO ((register char *));
 /* True if a string is all upper case. */
 extern int all_upper PROTO ((register char *));
 
-/* Bubble sort an integer array. */
-extern void bubble PROTO ((int[], int));
+/* Compare two integers for use by qsort. */
+extern int intcmp PROTO ((const void *, const void *));
 
 /* Check a character to make sure it's in the expected range. */
 extern void check_char PROTO ((int c));
@@ -840,8 +863,8 @@ extern char *copy_string PROTO ((register const char *));
 /* Returns a dynamically allocated copy of a (potentially) unsigned string. */
 extern Char *copy_unsigned_string PROTO ((register Char *));
 
-/* Shell sort a character array. */
-extern void cshell PROTO ((Char[], int, int));
+/* Compare two characters for use by qsort with '\0' sorting last. */
+extern int cclcmp PROTO ((const void *, const void *));
 
 /* Finish up a block of data declarations. */
 extern void dataend PROTO ((void));
@@ -885,6 +908,9 @@ extern void lerrif PROTO ((const char *, int));
 /* Report an error message formatted with one string argument. */
 extern void lerrsf PROTO ((const char *, const char *));
 
+/* Like lerrsf, but also exit after displaying message. */
+extern void lerrsf_fatal PROTO ((const char *, const char *));
+
 /* Spit out a "#line" statement. */
 extern void line_directive_out PROTO ((FILE *, int));
 
@@ -915,7 +941,6 @@ extern void out PROTO ((const char *));
 extern void out_dec PROTO ((const char *, int));
 extern void out_dec2 PROTO ((const char *, int, int));
 extern void out_hex PROTO ((const char *, unsigned int));
-extern void out_line_count PROTO ((const char *));
 extern void out_str PROTO ((const char *, const char *));
 extern void out_str3
 PROTO ((const char *, const char *, const char *, const char *));
@@ -1179,4 +1204,23 @@ int regmatch_len (regmatch_t * m);
 int regmatch_strtol (regmatch_t * m, const char *src, char **endptr, int base);
 bool regmatch_empty (regmatch_t * m);
 
+/* From "scanflags.h" */
+typedef unsigned int scanflags_t;
+extern scanflags_t* _sf_stk;
+extern size_t _sf_top_ix, _sf_max; /**< stack of scanner flags. */
+#define _SF_CASE_INS   0x0001
+#define _SF_DOT_ALL    0x0002
+#define _SF_SKIP_WS    0x0004
+#define sf_top()           (_sf_stk[_sf_top_ix])
+#define sf_case_ins()      (sf_top() & _SF_CASE_INS)
+#define sf_dot_all()       (sf_top() & _SF_DOT_ALL)
+#define sf_skip_ws()       (sf_top() & _SF_SKIP_WS)
+#define sf_set_case_ins(X)      ((X) ? (sf_top() |= _SF_CASE_INS) : (sf_top() &= ~_SF_CASE_INS))
+#define sf_set_dot_all(X)       ((X) ? (sf_top() |= _SF_DOT_ALL)  : (sf_top() &= ~_SF_DOT_ALL))
+#define sf_set_skip_ws(X)       ((X) ? (sf_top() |= _SF_SKIP_WS)  : (sf_top() &= ~_SF_SKIP_WS))
+extern void sf_init(void);
+extern void sf_push(void);
+extern void sf_pop(void);
+
+
 #endif /* not defined FLEXDEF_H */
