@@ -49,7 +49,7 @@ static char *basename2 PROTO ((char *path, int should_strip_ext));
 
 /* these globals are all defined and commented in flexdef.h */
 int     printstats, syntaxerror, eofseen, ddebug, trace, nowarn, spprdflt;
-int     interactive, caseins, lex_compat, posix_compat, do_yylineno,
+int     interactive, lex_compat, posix_compat, do_yylineno,
 	useecs, fulltbl, usemecs;
 int     fullspd, gen_line_dirs, performance_report, backing_up_report;
 int     C_plus_plus, long_align, use_read, yytext_is_array, do_yywrap,
@@ -57,7 +57,7 @@ int     C_plus_plus, long_align, use_read, yytext_is_array, do_yywrap,
 int     reentrant, bison_bridge_lval, bison_bridge_lloc;
 int     yymore_used, reject, real_reject, continued_action, in_rule;
 int     yymore_really_used, reject_really_used;
-int     datapos, dataline, linenum, out_linenum;
+int     datapos, dataline, linenum;
 FILE   *skelfile = NULL;
 int     skel_ind = 0;
 char   *action_array;
@@ -65,7 +65,7 @@ int     action_size, defs1_offset, prolog_offset, action_offset,
 	action_index;
 char   *infilename = NULL, *outfilename = NULL, *headerfilename = NULL;
 int     did_outfilename;
-char   *prefix, *yyclass;
+char   *prefix, *yyclass, *extra_type = NULL;
 int     do_stdinit, use_stdout;
 int     onestate[ONE_STACK_SIZE], onesym[ONE_STACK_SIZE];
 int     onenext[ONE_STACK_SIZE], onedef[ONE_STACK_SIZE], onesp;
@@ -136,6 +136,9 @@ extern FILE* yyout;
 static char outfile_path[MAXLINE];
 static int outfile_created = 0;
 static char *skelname = NULL;
+static int _stdout_closed = 0; /* flag to prevent double-fclose() on stdout. */
+const char *escaped_qstart = "[[]]M4_YY_NOOP[M4_YY_NOOP[M4_YY_NOOP[[]]";
+const char *escaped_qend   = "[[]]M4_YY_NOOP]M4_YY_NOOP]M4_YY_NOOP[[]]";
 
 /* For debugging. The max number of filters to apply to skeleton. */
 static int preproc_level = 1000;
@@ -147,7 +150,7 @@ int flex_main (argc, argv)
      int argc;
      char   *argv[];
 {
-	int     i, exit_status;
+	int     i, exit_status, child_status;
 
 	/* Set a longjmp target. Yes, I know it's a hack, but it gets worse: The
 	 * return value of setjmp, if non-zero, is the desired exit code PLUS ONE.
@@ -158,17 +161,29 @@ int flex_main (argc, argv)
 	 */
 	exit_status = setjmp (flex_main_jmp_buf);
 	if (exit_status){
-        fflush(stdout);
-        fclose(stdout);
-        while (wait(0) > 0){
+        if (stdout && !_stdout_closed && !ferror(stdout)){
+            fflush(stdout);
+            fclose(stdout);
         }
-		return exit_status - 1;
+        while (wait(&child_status) > 0){
+            if (!WIFEXITED (child_status)
+                || WEXITSTATUS (child_status) != 0){
+                /* report an error of a child
+                 */
+                if( exit_status <= 1 )
+                    exit_status = 2;
+
+            }
+        }
+        return exit_status - 1;
     }
 
 	flexinit (argc, argv);
 
 	readin ();
 
+	skelout ();
+	/* %% [1.5] DFA */
 	ntod ();
 
 	for (i = 1; i <= num_rules; ++i)
@@ -198,9 +213,12 @@ int main (argc, argv)
      char   *argv[];
 {
 #if ENABLE_NLS
+#if HAVE_LOCALE_H
 	setlocale (LC_MESSAGES, "");
+        setlocale (LC_CTYPE, "");
 	textdomain (PACKAGE);
 	bindtextdomain (PACKAGE, LOCALEDIR);
+#endif
 #endif
 
 	return flex_main (argc, argv);
@@ -211,6 +229,7 @@ int main (argc, argv)
 void check_options ()
 {
 	int     i;
+    const char * m4 = NULL;
 
 	if (lex_compat) {
 		if (C_plus_plus)
@@ -223,11 +242,6 @@ void check_options ()
 			flexerror (_
 				   ("Can't use --reentrant or --bison-bridge with -l option"));
 
-		/* Don't rely on detecting use of yymore() and REJECT,
-		 * just assume they'll be used.
-		 */
-		yymore_really_used = reject_really_used = true;
-
 		yytext_is_array = true;
 		do_yylineno = true;
 		use_read = false;
@@ -317,6 +331,9 @@ void check_options ()
     if (!ansi_func_protos)
         buf_m4_define( &m4defs_buf, "M4_YY_NO_ANSI_FUNC_PROTOS", NULL);
 
+    if (extra_type)
+        buf_m4_define( &m4defs_buf, "M4_EXTRA_TYPE_DEFS", extra_type);
+
 	if (!use_stdout) {
 		FILE   *prev_stdout;
 
@@ -328,7 +345,7 @@ void check_options ()
 			else
 				suffix = "c";
 
-			sprintf (outfile_path, outfile_template,
+			snprintf (outfile_path, sizeof(outfile_path), outfile_template,
 				 prefix, suffix);
 
 			outfilename = outfile_path;
@@ -342,9 +359,12 @@ void check_options ()
 		outfile_created = 1;
 	}
 
+
     /* Setup the filter chain. */
     output_chain = filter_create_int(NULL, filter_tee_header, headerfilename);
-    filter_create_ext(output_chain,"m4","-P",0);
+    if ( !(m4 = getenv("M4")))
+        m4 = M4;
+    filter_create_ext(output_chain, m4, "-P", 0);
     filter_create_int(output_chain, filter_fix_linedirs, NULL);
 
     /* For debugging, only run the requested number of filters. */
@@ -374,11 +394,9 @@ void check_options ()
 		buf_m4_define (&m4defs_buf, "M4_YY_TABLES_EXTERNAL", NULL);
 
 		if (!tablesfilename) {
-			nbytes = strlen (prefix) +
-				strlen (tablesfile_template) + 2;
-			tablesfilename = pname =
-				(char *) calloc (nbytes, 1);
-			sprintf (pname, tablesfile_template, prefix);
+			nbytes = strlen (prefix) + strlen (tablesfile_template) + 2;
+			tablesfilename = pname = (char *) calloc (nbytes, 1);
+			snprintf (pname, nbytes, tablesfile_template, prefix);
 		}
 
 		if ((tablesout = fopen (tablesfilename, "w")) == NULL)
@@ -391,7 +409,7 @@ void check_options ()
 
 		nbytes = strlen (prefix) + strlen ("tables") + 2;
 		tablesname = (char *) calloc (nbytes, 1);
-		sprintf (tablesname, "%stables", prefix);
+		snprintf (tablesname, nbytes, "%stables", prefix);
 		yytbl_hdr_init (&hdr, flex_version, tablesname);
 
 		if (yytbl_hdr_fwrite (&tableswr, &hdr) <= 0)
@@ -431,9 +449,12 @@ void check_options ()
         buf_init(&tmpbuf, sizeof(char));
         for (i = 1; i <= lastsc; i++) {
              char *str, *fmt = "#define %s %d\n";
-             
-             str = (char*)flex_alloc(strlen(fmt) + strlen(scname[i]) + (int)(1 + log(i)/log(10)) + 2);
-             sprintf(str, fmt,      scname[i], i - 1);
+             size_t strsz;
+
+             str = (char*)flex_alloc(strsz = strlen(fmt) + strlen(scname[i]) + (int)(1 + log10(i)) + 2);
+             if (!str)
+               flexfatal(_("allocation of macro definition failed"));
+             snprintf(str, strsz, fmt,      scname[i], i - 1);
              buf_strappend(&tmpbuf, str);
              free(str);
         }
@@ -453,7 +474,7 @@ void check_options ()
 
     /* Place a bogus line directive, it will be fixed in the filter. */
     outn("#line 0 \"M4_YY_OUTFILE_NAME\"\n");
-    
+
 	/* Dump the user defined preproc directives. */
 	if (userdef_buf.elts)
 		outn ((char *) (userdef_buf.elts));
@@ -488,7 +509,7 @@ void flexend (exit_status)
 				skelname);
 	}
 
-#if 0 
+#if 0
 		fprintf (header_out,
 			 "#ifdef YY_HEADER_EXPORT_START_CONDITIONS\n");
 		fprintf (header_out,
@@ -622,6 +643,7 @@ void flexend (exit_status)
                 "yylex",
                 "yylex_destroy",
                 "yylex_init",
+                "yylex_init_extra",
                 "yylineno",
                 "yylloc",
                 "yylval",
@@ -646,7 +668,7 @@ void flexend (exit_status)
                 /* must be null-terminated */
                 NULL};
 
-                
+
                 for (i=0; undef_list[i] != NULL; i++)
                     fprintf (header_out, "#undef %s\n", undef_list[i]);
         }
@@ -679,7 +701,7 @@ void flexend (exit_status)
 			lerrsf (_("error writing output file %s"),
 				outfilename);
 
-		else if (fclose (stdout))
+		else if ((_stdout_closed = 1) && fclose (stdout))
 			lerrsf (_("error closing output file %s"),
 				outfilename);
 
@@ -722,7 +744,7 @@ void flexend (exit_status)
 			putc ('b', stderr);
 		if (ddebug)
 			putc ('d', stderr);
-		if (caseins)
+		if (sf_case_ins())
 			putc ('i', stderr);
 		if (lex_compat)
 			putc ('l', stderr);
@@ -911,7 +933,7 @@ void flexinit (argc, argv)
 	char   *arg;
 	scanopt_t sopt;
 
-	printstats = syntaxerror = trace = spprdflt = caseins = false;
+	printstats = syntaxerror = trace = spprdflt = false;
 	lex_compat = posix_compat = C_plus_plus = backing_up_report =
 		ddebug = fulltbl = false;
 	fullspd = long_align = nowarn = yymore_used = continued_action =
@@ -953,6 +975,8 @@ void flexinit (argc, argv)
         buf_append (&m4defs_buf, &m4defs_init_str, 2);
     }
 
+    sf_init ();
+
     /* initialize regex lib */
     flex_init_regex();
 
@@ -1068,7 +1092,7 @@ void flexinit (argc, argv)
 			break;
 
 		case OPT_CASE_INSENSITIVE:
-			caseins = true;
+			sf_set_case_ins(true);
 			break;
 
 		case OPT_LEX_COMPAT:
@@ -1416,7 +1440,7 @@ void flexinit (argc, argv)
 	num_backing_up = onesp = numprots = 0;
 	variable_trailing_context_rules = bol_needed = false;
 
-	out_linenum = linenum = sectnum = 1;
+	linenum = sectnum = 1;
 	firstprot = NIL;
 
 	/* Used in mkprot() so that the first proto goes in slot 1
@@ -1551,7 +1575,11 @@ void readin ()
     }
 
 	if (!do_yywrap) {
-		outn ("\n#define yywrap(n) 1");
+		if (!C_plus_plus)
+			 if (reentrant)
+				outn ("\n#define yywrap(yyscanner) 1");
+			 else
+				outn ("\n#define yywrap() 1");
 		outn ("#define YY_SKIP_YYWRAP");
 	}
 
@@ -1595,7 +1623,6 @@ void readin ()
 			outn ("#else");
 			outn (yy_stdinit);
 			outn ("#endif");
-			outn ("#endif");
 		}
 
 		else {
@@ -1625,6 +1652,10 @@ void readin ()
 	if (C_plus_plus) {
 		outn ("\n#include <FlexLexer.h>");
 
+ 		if (!do_yywrap) {
+			outn("\nint yyFlexLexer::yywrap() { return 1; }");
+		}
+
 		if (yyclass) {
 			outn ("int yyFlexLexer::yylex()");
 			outn ("\t{");
@@ -1760,7 +1791,7 @@ void usage ()
 	FILE   *f = stdout;
 
 	if (!did_outfilename) {
-		sprintf (outfile_path, outfile_template,
+		snprintf (outfile_path, sizeof(outfile_path), outfile_template,
 			 prefix, C_plus_plus ? "cc" : "c");
 		outfilename = outfile_path;
 	}
